/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "TextMetrics.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomTextMetrics.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomTextMetrics.h"
#include "gdom/PrivateIntGdomTextMetrics.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::TextMetrics* obj)
{
    return WebCore::cacheDOMObject<WebCore::TextMetrics, GDOMTextMetrics>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMTextMetrics::GDOMTextMetrics(PassRefPtr<WebCore::TextMetrics> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMTextMetrics::~GDOMTextMetrics()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "TextMetrics.h"
#include <gdom/GdomTextMetrics.h>
#include <gdom/PrivateGdomTextMetrics.h>
#include <gdom/PrivateIntGdomTextMetrics.h>
#include <gdom/GdomTextMetricsprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomTextMetrics, gdom_text_metrics, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomTextMetricsPrivate {
    class WebCore::TextMetrics *item;
};

#define GDOM_TEXT_METRICS_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_TEXT_METRICS, GdomTextMetricsPrivate))
*/
G_END_DECLS
WebCore::TextMetrics *
gdom_text_metrics_get_private(GdomTextMetrics *request)
{
    GDOMTextMetrics *res;
    res = static_cast<GDOMTextMetrics*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::TextMetrics*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_WIDTH,
};


static void gdom_text_metrics_finalize(GObject* object)
{
    /*GdomTextMetrics* request = GDOM_TEXT_METRICS(object);*/
    /*GdomTextMetricsPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_text_metrics_parent_class)->finalize(object);
}

static void gdom_text_metrics__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomTextMetrics *self = GDOM_TEXT_METRICS(object);
    WebCore::TextMetrics * gself = (gdom_text_metrics_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_text_metrics__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomTextMetrics *self = GDOM_TEXT_METRICS(object);
    WebCore::TextMetrics * gself = (gdom_text_metrics_get_private(self));
    switch (prop_id) {
    case PROP_WIDTH:
    {
         g_value_set_float (value, gself->width());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_text_metrics_class_init(GdomTextMetricsClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_text_metrics_finalize;
    g_klass->set_property = gdom_text_metrics__set_property_fn;
    g_klass->get_property = gdom_text_metrics__get_property_fn;

    g_object_class_install_property (g_klass, PROP_WIDTH,
            g_param_spec_float ("width", /* name */
                "text_metrics_width", /* short description */
                "read-only  gfloat TextMetrics.width", /* longer - could do with some extra doc stuff here */
                 G_MINFLOAT,        /* min */
                G_MAXFLOAT, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomTextMetricsPrivate));*/
}

static void gdom_text_metrics_init(GdomTextMetrics* request)
{
    /*GdomTextMetricsPrivate* priv = GDOM_TEXT_METRICS_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

